﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ServiceModel;

namespace MainServiceApp
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.Title = "主示例";
            using (ServiceHost host = new ServiceHost(typeof(TestService)))
            {
                host.AddServiceEndpoint(typeof(ITestService), new WSHttpBinding(), "http://localhost:9000");
                host.Open(); //运行服务
                Console.WriteLine("主服务已启动。");

                // 客户端调用
                Action actdlg = () =>
                  {
                      ITestService cn = ChannelFactory<ITestService>.CreateChannel(new WSHttpBinding(), new EndpointAddress("http://localhost:9000"));
                      cn.TestCall();
                      // 关闭通道
                      ((IClientChannel)cn).Close();
                  };
                // 开启5个任务
                Task[] tasks = new Task[5];
                // 初始化每个任务
                for (int n = 0; n < tasks.Length; n++)
                {
                    tasks[n] = new Task(actdlg);
                }
                // 开始执行任务
                foreach (Task t in tasks)
                {
                    t.Start();
                }
                // 等待所有任务完成
                Task.WaitAll(tasks);

                Console.ReadKey();
            }
        }
    }


    [ServiceContract]
    public interface ITestService
    {
        [OperationContract]
        void TestCall();
    }

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, ConcurrencyMode = ConcurrencyMode.Reentrant)]
    internal class TestService : ITestService
    {
        public void TestCall()
        {
            Console.WriteLine("即将调用另一个服务。");
            // 调用其他服务
            ISome cnl = ChannelFactory<ISome>.CreateChannel(new BasicHttpBinding(), new EndpointAddress("http://localhost:12345"));
            // 调用完成后，再次回来
            Guid id = cnl.GetUID();
            ((IClientChannel)cnl).Close();
            Console.WriteLine($"回到当前服务。得到的ID为：{id}");
        }
    }

    [ServiceContract]
    public interface ISome
    {
        [OperationContract]
        Guid GetUID();
    }
}
